/*
 * The Syncro Soft SRL License
 * 
 * 
 * Copyright (c) 1998-2002 Syncro Soft SRL, Romania. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted
 * provided that the following conditions are met: 1. Redistribution of source or in binary form is
 * allowed only with the prior written permission of Syncro Soft SRL. 2. Redistributions of source
 * code must retain the above copyright notice, this list of conditions and the following
 * disclaimer. 2. Redistributions in binary form must reproduce the above copyright notice, this
 * list of conditions and the following disclaimer in the documentation and/or other materials
 * provided with the distribution. 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment: "This product includes software developed by
 * the Syncro Soft SRL (http://www.sync.ro/)." Alternately, this acknowledgment may appear in the
 * software itself, if and wherever such third-party acknowledgments normally appear. 4. The names
 * "Oxygen" and "Syncro Soft SRL" must not be used to endorse or promote products derived from this
 * software without prior written permission. For written permission, please contact
 * contact@sync.ro. 5. Products derived from this software may not be called "Oxygen", nor may
 * "Oxygen" appear in their name, without prior written permission of the Syncro Soft SRL.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.oxygenxml.editor.sample.extension.action;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.VerifyKeyListener;
import org.eclipse.swt.events.VerifyEvent;

import com.oxygenxml.editor.editors.xml.extension.TextExtensionAction;
import com.oxygenxml.editor.sample.extension.SamplePlugin;

/**
 * Extension action for locking/unlocking XML tag editing in oXygen's XML editor. 
 * The action will be added to the toolbar associated with the XML editor and
 * is configured in plugin.xml.
 *
 * @version $Revision: 1.1 $
 */
public class LockUnlockTagsAction extends TextExtensionAction implements IExecutableExtension {

  /**
   * Name of attribute in plugin.xml with display action name.
   */
  private final static String TOOLBAR_ACTIONS_NAME_ATTRIB_NAME = "label";

  /**
   * Name of attribute in plugin.xml with action icon for locked status.
   */
  private final static String TOOLBAR_ACTIONS_LOCK_ICON_ATTRIB_NAME = "lockIcon";

  /**
   * Name of attribute in plugin.xml with action icon for unlocked status.
   */
  private final static String TOOLBAR_ACTIONS_UNLOCK_ICON_ATTRIB_NAME = "unlockIcon";

  /**
   * True when tag editing is locked.
   */
  private boolean locked;

  /**
   * Name of file with icon for locked state.
   */
  private String lockIconName;

  /**
   * Name of file with icon for unlocked state.
   */
  private String unlockIconName;

  /**
   * Sets the extension parameters.
   * 
   * @param configElement The configuration element.
   * @param attrName The attribute name.
   * @param initData The init data.
   * @throws CoreException when the initialization fails.
   */
  public void setInitializationData(IConfigurationElement configElement, String attrName, Object initData)
    throws CoreException {
    String text = configElement.getAttribute(TOOLBAR_ACTIONS_NAME_ATTRIB_NAME);
    setText(text);
    setToolTipText(text);
    lockIconName = configElement.getAttribute(TOOLBAR_ACTIONS_LOCK_ICON_ATTRIB_NAME);
    unlockIconName = configElement.getAttribute(TOOLBAR_ACTIONS_UNLOCK_ICON_ATTRIB_NAME);
    updateIcon();
  }

  /**
   * Update action icon depending on current locked state.
   */
  private void updateIcon() {
    setImageDescriptor(
      SamplePlugin.imageDescriptorFromPlugin(
        SamplePlugin.PLUGIN_ID,
        locked ? lockIconName : unlockIconName));
  }

  /**
   * The listener that blocks XML tag modifications.
   */
  private VerifyKeyListener listener = new VerifyKeyListener() {
    public void verifyKey(VerifyEvent event) {
      if (!Character.isISOControl(event.character)
        || Character.isWhitespace(event.character)
        || event.character == SWT.BS
        || event.character == SWT.DEL) {
        int pos = styledText.getCaretOffset();
        String before = styledText.getText(0, pos);
        if (before.lastIndexOf(">") < before.lastIndexOf("<")) {
          event.doit = false;
        }
      }
    }
  };

  /**
   * Toggles the locked state of tag editing and updates the action icon.
   */
  public void run() {
    locked = !locked;
    if (locked) {
      styledText.addVerifyKeyListener(listener);
    } else {
      styledText.removeVerifyKeyListener(listener);
    }
    updateIcon();
  }
}
