/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.css;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.annotations.obfuscate.SkipLevel;
import ro.sync.annotations.obfuscate.SkipObfuscate;

/**
 * A length that may be expressed as an absolute or relative value, or as an "auto" value, that
 * is to be computed at later time, by the layout engine.
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PRIVATE)
@SkipObfuscate(classes = SkipLevel.NOT_SPECIFIED, fields = SkipLevel.NOT_SPECIFIED, methods = SkipLevel.PUBLIC)
public class RelativeLength {
  /**
   * Relative length expressed as percentage or absolute value.
   */
  private float value;
  
  /**
   * It represents an absolute length.
   */
  private static final byte LENGTH_ABSOLUTE = 0; 

  /**
   * It represents a relative length.
   */
  private static final byte LENGTH_RELATIVE = 1;
  
  /**
   * It represents an "auto" length.
   */
  private static final byte LENGTH_AUTO = 2; 
  
  /**
   * The length type. Can be one of the constants: LENGTH_ABSOLUTE, LENGTH_RELATIVE, LENGTH_AUTO.
   */
  private byte type; 
  
  /**
   * Cache of absolute values
   */
  private static final RelativeLength[] CACHED_ABSOLUTE = new RelativeLength[500];

  /**
   * We need only one instance of the "auto" length.
   */
  private static final RelativeLength CACHED_AUTO = new RelativeLength(0, LENGTH_AUTO);
  
  static {
    for (int i = 0; i < CACHED_ABSOLUTE.length; i++) {
      CACHED_ABSOLUTE[i] = new RelativeLength(i, LENGTH_ABSOLUTE);
    }
  }
  
  /**
   * Create an absolute value length.
   * 
   * @param value The value of the absolute length.
   * @return The new absolute value RelativeLength.
   */
  public static RelativeLength createAbsolute(int value) {
    if (value >=0 && value < CACHED_ABSOLUTE.length) {
      return CACHED_ABSOLUTE[value];
    } else {
      return new RelativeLength(value, LENGTH_ABSOLUTE);
    }
  }

  /**
   * Create a relative length representing a relative value.
   * 
   * @param percentage The percentage from the value this length will refer to. (e.g: 50%)
   * @return The new relative value RelativeLength.
   */
  public static RelativeLength createRelative(float percentage) {
    return new RelativeLength(percentage, LENGTH_RELATIVE);
  }
  

  /**
   * Create an automatic relative length. This is just an immutable constant marker.
   * 
   * @return The "auto" relative length. Always the same object.
   */
  public static RelativeLength createAuto() {
    return CACHED_AUTO;
  }
  
  
  /**
   * Return the evaluated value of the RelativeLength given a reference value. If this
   * object represents an absolute value, that value is simply returned.
   * Otherwise, returns the given reference length multiplied by the given
   * percentage divided to 100 and rounded to the nearest integer.
   * 
   * @param referenceLength Reference length for percentage lengths.
   * @return The actual value.
   */
  public int get(int referenceLength) {
    if (type == LENGTH_ABSOLUTE || type == LENGTH_AUTO) {
      return (int) this.value;
    } else {
      return Math.round((this.value * referenceLength) / 100);
    }
  }

  /**
   * Constructor
   * @param value The absolute or percentage value.
   * @param type Indicates if the value is absolute, relative or "auto".
   */
  private RelativeLength(float value, byte type) {
    this.value = value;
    this.type = type;
  }
  
  /**
   * @see java.lang.Object#hashCode()
   */
  @Override
  public int hashCode() {
    int hash = 0;
    hash = type;
    hash += new Float(value).hashCode();
    return hash;
  }
  
  /**
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object obj) {
    if (obj instanceof RelativeLength) {
      RelativeLength rl = (RelativeLength) obj;
      return (type == rl.type) && (value == rl.value);
    }
    return false;
  }
  /**
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString() {
    return "[Relative Length: " + value + " Type: " + type + " ]";
  }

  /**
   * @return True if the relative length is automatic.
   */
  public boolean isAuto() {
    return type == LENGTH_AUTO;
  }
 
  /**
   * @return True if the length is relative.
   */
  public boolean isRelative() {
    return type == LENGTH_RELATIVE;
  }
  
  /**
   * @return True if the length is relative.
   */
  public boolean isAbsolute() {
    return type == LENGTH_ABSOLUTE;
  }
  
  /**
   * @return The value of the absolute length or the percentage of a relative 
   * length.
   */
  public float getValue() {
    return this.value;
  }
}
