/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.css;

import java.net.MalformedURLException;
import java.net.URL;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.annotations.obfuscate.SkipLevel;
import ro.sync.annotations.obfuscate.SkipObfuscate;
import ro.sync.util.Equaler;
import ro.sync.util.URLUtil;
import ro.sync.xml.catalogresolver.CatalogResolverFactory;
import ro.sync.xml.catalogresolver.ResourceURIResolver;

/**
 * URI content
 */
@API(type=APIType.INTERNAL, src=SourceType.PRIVATE)
@SkipObfuscate(classes = SkipLevel.NOT_SPECIFIED, fields = SkipLevel.NOT_SPECIFIED, methods = SkipLevel.PUBLIC)
public class URIContent implements StaticContent {
  
  /**
   * The URI
   */
  private final String href;
  
  /**
   * Base system ID
   */
  private final String base;

  /**
   * Constructor
   * @param base Base system ID.
   * @param href Href. 
   */
  public URIContent(String base, String href) {
    this.base = base;
    this.href = href;
  }

  /**
   * @see ro.sync.ecss.css.StaticContent#getType()
   */
  @Override
  public int getType() {
    return URI_CONTENT;
  }

  /**
   * @return The href.
   */
  public String getHref() {
    return href;
  }

  /**
   * @return The Base URL
   */
  public String getBase() {
    return base;
  }
  
  /**
   * @see java.lang.Object#hashCode()
   */
  @Override
  public int hashCode() {
    int hash = 0;
    if (base != null) {
      hash = base.intern().hashCode();
    }
    if (href != null) {
      hash += href.intern().hashCode();
    }
    return hash;
  }
  
  /**
   * @see java.lang.Object#equals(java.lang.Object)
   */
  @Override
  public boolean equals(Object obj) {
    if (obj instanceof URIContent) {
      URIContent uc = (URIContent) obj;
      return Equaler.verifyEquals(base, uc.base) && 
          Equaler.verifyEquals(href, uc.href);
    }
    return false;
  }
  
  /**
   * Returns the image URL after resolving it with the catalog.
   * 
   * This methods should be used as a fallback for the extension bundle 
   * resolving. 
   * 
   * @return The expanded URL.
   * 
   * @throws MalformedURLException
   */
  public URL getCatalogExpandedURL() throws MalformedURLException {
    URL expanded = null;

    // Href not handled by the extension bundle.
    // EXM-12750, EXM-15943 Get the full URL and pass through catalog resolver.
    String correctedHref = URLUtil.correct(href, false);

    ////////////////
    // Catalog
    ////////////////
    ResourceURIResolver uriResolver = CatalogResolverFactory.getResourceURIResolver();

    // 1. Try to solve the href or the base + href..                
    String solved = uriResolver.resolveURI(correctedHref, getBase());
    if (solved != null) {
      expanded = new URL(solved);
    }

    // 2. The fallback. Compose the expanded system id directly from the base and href.
    if (expanded == null) {
      String base = getBase();
      if(base == null) {
        expanded = new URL(correctedHref);
      } else {
        expanded = new URL(new URL(URLUtil.correct(base, false)), correctedHref);
      }
    }
    return expanded;
  }
}