/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.content;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.AuthorSchemaAwareEditingHandler;
import ro.sync.ecss.extensions.api.node.AuthorDocumentFragment;

/**
 * Provides information about a fragment in the clipboard.
 * 
 * The fragment received in the callback can either be directly processed or a new fragment can be set instead.
 * An easy way to modify the fragment XML content is like this:<br>
 * <pre>
 *  public void process(ClipboardFragmentInformation fragmentInformation) {
 *   AuthorDocumentFragment frag = fragmentInformation.getFragment();
 *   try {
 *     String xmlContent = authorAccess.getDocumentController().serializeFragmentToXML(frag);
 *     //Modify the XML content here.....
 *     
 *     //Re-create a new fragment from the XML
 *     AuthorDocumentFragment newFragment = authorAccess.getDocumentController().createNewDocumentFragmentInContext(xmlContent, authorAccess.getEditorAccess().getCaretOffset());
 *     
 *     fragmentInformation.setFragment(newFragment);
 *   } catch (BadLocationException e) {
 *     e.printStackTrace();
 *   } catch (AuthorOperationException e) {
 *     e.printStackTrace();
 *   }
 * }</pre>
 * 
 * @since 12.2
 */
@API(type=APIType.NOT_EXTENDABLE, src=SourceType.PUBLIC)
public interface ClipboardFragmentInformation {

  /**
   * Get the purpose for which this fragment is used. One of the constants:
   * <ul>
   *  <li>{@link AuthorSchemaAwareEditingHandler#CREATE_FRAGMENT_PURPOSE_COPY}
   *  <li>{@link AuthorSchemaAwareEditingHandler#CREATE_FRAGMENT_PURPOSE_CUT}
   *  <li>{@link AuthorSchemaAwareEditingHandler#CREATE_FRAGMENT_PURPOSE_DND_COPY}
   *  <li>{@link AuthorSchemaAwareEditingHandler#CREATE_FRAGMENT_PURPOSE_DND_MOVE}
   * </ul>
   *
   * @return The purpose for which this fragment is used.
   */
  int getPurposeID();
  
  /**
   * Get an unique ID of this object taken from the clipboard.
   * For example if multiple pastes are made with same content, the same object ID is returned.
   * 
   * @return An unique ID of this object taken from the clipboard.
   */
  String getClipboardObjectID();
  
  /**
   * Get the URL of the file from which the fragment was copied.
   * 
   * @return The URL of the file from which the fragment was copied.
   */
  String getFragmentOriginalLocation();
  
  /**
   * Get the fragment which will be pasted/dropped.
   * 
   * @return The fragment which will be pasted/dropped.
   */
  AuthorDocumentFragment getFragment();
  
  /**
   * Set a new fragment replacing the original one which will be pasted/dropped.
   * 
   * @param newFragment The fragment which will be pasted/dropped.
   * 
   * @since 13
   */
  void setFragment(AuthorDocumentFragment newFragment);
}