/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.api.editor;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.extensions.api.CursorType;
import ro.sync.ecss.extensions.api.Extension;
import ro.sync.ecss.extensions.commons.editor.SimpleURLChooserEditor;

/**
 * An author in-place renderer. Subclasses contain implementation based on real
 * SWT or Swing components like combo boxes, text fields, check boxes, buttons etc.
 * 
 * <p>A renderer is associated with an oxy_editor CSS function and it's the instance
 * that handles the painting of the value that the {@link InplaceEditor} will edit.</p>
 * 
 * <p>It is recommended to extend the adapter class {@link InplaceEditorRendererAdapter}
 * or {@link InplaceRendererAdapter} in order to be protected from future API additions.</p>
 * 
 * <p>
 * A renderer of a certain type will be reused to render in different contexts so the 
 * renderer must initialize itself for the given {@link AuthorInplaceContext} on 
 * each of its called methods.
 * 
 * Example: For the following CSS, the same <b>combo</b> renderer (same instance) will be used for both
 * attributes. So for each callback it must prepare itself for the current context.
 * <pre>
 * code {
 *   content: 
 *     "Attr1: " oxy_editor(type, combo, edit, '@attr1')
 *     "Attr2: " oxy_editor(type, combo, edit, '@attr2')
 *  }
 * </pre>
 * </p>
 * <p> 
 * For a custom implementation property {@link InplaceEditorCSSConstants#PROPERTY_RENDERER_CLASS_NAME}
 * must be used. The same implementation will be used in Standalone or Eclipse plugin
 * environment:
 * <pre>
 * myElement {
 *   content: oxy_editor(
 *       rendererClassName, "com.custom.editors.CustomRenderer",
 *       swingEditorClassName, "com.custom.editors.SwingCustomEditor",
 *       swtEditorClassName, "com.custom.editors.SwtCustomEditor",
 *       edit, "@my_attr"
 *       customProperty1, "customValue1",
 *       customProperty2, "customValue2"
 *   )
 * }
 * </pre>
 * In the previous example, the renderer class com.custom.editors.CustomRenderer 
 * must be added in the Classpath of the document type associated with the edited document.
 * </p>
 * 
 * @see SimpleURLChooserEditor
 * 
 * @since 14.1
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public interface InplaceRenderer extends Extension {
  /**
   * Initialize the renderer with the given context and returns the component.
   * It's up to the caller to use the renderer to paint.
   * 
   * @param context The editing context.
   * 
   * @return The renderer. A java.awt.Component implementation.
   */
  Object getRendererComponent(AuthorInplaceContext context);
  
  /**
   * Get a cursor to be used when the user hovers with the mouse over this renderer.
   * For a more complex renderer, the given X,Y coordinates can be used to decide
   * what cursor to return.
   * 
   * @param context The editing context. Useful if the renderer is a more complex one, 
   * like a text field with an associated button and wants to provide different 
   * cursors when the cursor is over the textfield or over the button. In this case 
   * the renderer will have to initialize itself with this context in order to 
   * decide what the cursor is hovering.
   *  
   * @param x The x coordinate relative to the renderer bounds.
   * @param y The y coordinate relative to the renderer bounds.
   * 
   * @return The type of cursor to be used or <code>null</code> to let the viewport decide.
   */
  CursorType getCursorType(AuthorInplaceContext context, int x, int y);
  
  /**
   * Get a cursor to be used when the user hovers with the mouse over this renderer.
   * For a more complex renderer, the given X,Y coordinates can be used to decide
   * what cursor to return.
   * 
   * We recommend using {@link #getCursorType(AuthorInplaceContext, int, int)} as 
   * you can use the provided context to get additional information.
   * 
   * @param x The x coordinate relative to the renderer bounds.
   * @param y The y coordinate relative to the renderer bounds.
   * 
   * @return The type of cursor to be used or <code>null</code> to let the viewport decide.
   */
  CursorType getCursorType(int x, int y);
  
  /**
   * Gets a tooltip text to be presented when the cursor is over this renderer.
   * <b>Because a renderer is reused, when this 
   * called is received, the renderer must re-initialize itself from the given context.</b>
   * 
   * @param context The editing context.
   * @param x The x coordinate relative to the renderer bounds.
   * @param y The y coordinate relative to the renderer bounds.
   * 
   * @return A tooltip text or <code>null</code> if no tooltip.
   */
  String getTooltipText(AuthorInplaceContext context, int x, int y);
  
  /**
   * Returns the rendering layout info. This contains information about the baseline
   * and the size in a certain context. The baseline is measured from the top of 
   * the component.
   * 
   * <b>Because a renderer is reused, when this 
   * call is received, the renderer must re-initialize itself from the given context.</b>
   * 
   * @param context The editing context.
   * 
   * @return The rendering layout info.
   */
  RendererLayoutInfo getRenderingInfo(AuthorInplaceContext context);
}