/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.view.graphics;

import java.util.StringTokenizer;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.annotations.obfuscate.SkipLevel;
import ro.sync.annotations.obfuscate.SkipObfuscate;
import ro.sync.util.NumberParserUtil;
import ro.sync.util.NumberFormatException;

/**
 * The class used to represent a Color.
 * 
 * WARNING: This class is immutable. Its values are sometimes cached in class StyleSheet.
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PRIVATE)
@SkipObfuscate(classes = SkipLevel.PRIVATE, fields = SkipLevel.PRIVATE, methods = SkipLevel.PRIVATE)
public class Color {
  /**
   * Light gray with alpha color constant. The RGB values are: (192, 192, 192, 128).
   */
  public static final Color COLOR_WHITE_ALPHA = new Color(255, 255, 255, 200);

  /**
   * Light gray color constant. The RGB values are: (192, 192, 192).
   */
  public static final Color COLOR_LIGHT_GRAY = new Color(192, 192, 192);
  
  /**
   * Gray with alpha color constant. The RGB values are: (192, 192, 192, 50).
   */
  public static final Color COLOR_LIGHT_GRAY_ALPHA = new Color(192, 192, 192, 50);

  /**
   * White color constant. The RGB values are: (255, 255, 255).
   */
  public static final Color COLOR_WHITE = new Color(255, 255, 255);

  /**
   * Black color constant. The RGB values are: (0, 0, 0).
   */
  public static final Color COLOR_BLACK = new Color(0, 0, 0);
  /**
   * Black color with alpha constant. The RGB values are: (0, 0, 0, 40).
   */
  public static final Color COLOR_BLACK_ALPHA = new Color(0, 0, 0, 40);

  /**
   * Red color constant. The RGB values are: (255, 0, 0).
   */
  public static final Color COLOR_RED = new Color(255, 0, 0);
  
  /**
   * Red darker color constant. The RGB values are: (178, 0, 0).
   */
  public static final Color COLOR_RED_DARKER = new Color(178, 0, 0);

  /**
   * Dark gray color constant. The RGB values are: (64, 64, 64).
   */
  public static final Color COLOR_DARK_GRAY = new Color(64, 64, 64);

  /**
   * Blue color constant. The RGB values are: (0, 0, 255).
   */
  public static final Color COLOR_BLUE = new Color(0, 0, 255);

  /**
   * Gray color constant. The RGB values are: (128, 128, 128).
   */
  public static final Color COLOR_GRAY = new Color(128, 128, 128);

  /**
   * Light green color constant. The RGB values are: (230, 255, 230).
   */
  public static final Color COLOR_LIGHT_GREEN = new Color(230, 255, 230);
  
  /**
   * Orange color constant. The RGB values are: (255, 200, 0).
   */
  public static final Color COLOR_ORANGE = new Color(255, 200, 0);

  /**
   * Lighter gray color constant. The RGB values are: (240, 240, 240).
   */
  public static final Color COLOR_LIGHTER_GRAY = new Color(240, 240, 240);
  
  /**
   * Lighter blue color constant. The RGB values are: (247, 250, 252).
   */
  public static final Color COLOR_LIGHTER_BLUE = new Color(247, 250, 252);

  /**
   * Darker green color constant. The RGB values are: (0, 128, 0).
   */  
  public static final Color COLOR_DARK_GREEN = new Color(0, 128, 0);
  
  /**
   * Light yellow color constant. The RGB values are: (255, 255, 230).
   */
  public static final Color COLOR_LIGHT_YELLOW = new Color(255, 255, 230);
  
  /**
   * A yellow color.
   */
  public static final Color COLOR_YELLOW = new Color(255,255,158);
  /**
   * A dark yellow color.
   */
  public static final Color COLOR_DARK_YELLOW = new Color(255,255,50);

  /**
   * The color RGB value.
   */
  final int rgb;

  /**
   * Creates an sRGB color with the specified red, green, blue, and alpha
   * values in the range (0 - 255).
   * 
   * @param r the red component.
   * @param g the green component.
   * @param b the blue component.
   * @param a the alpha component.
   */
  public Color(int r, int g, int b, int a) {
    this(getRGB(r, g, b, a));
  }

  /**
   * Get a RGB for a color.
   * 
   * @param red   Red value.
   * @param green Green value.
   * @param blue  Blue value
   * @param alpha Alpha value.
   *
   * @return The representation of the given values as an int.
   */
  public static int getRGB(int red, int green, int blue, int alpha) {
    return ((alpha & 0xFF) << 24) |
        ((red & 0xFF) << 16) |
        ((green & 0xFF) << 8)  |
        ((blue & 0xFF) << 0);
  }
  
  /**
   * Creates an sRGB color from the specified RGB color value.
   * 
   * @param rgb The value of the RGB color. Bits 24-31 are alpha, 16-23 are red, 8-15 are green, 
   * 0-7 are blue.
   */
  public Color(int rgb) {
    this.rgb = rgb;
  }

  /**
   * Creates an opaque sRGB color with the specified red, green, 
   * and blue values in the range (0 - 255).
   * The actual color used in rendering depends
   * on finding the best match given the color space 
   * available for a given output device.
   * Alpha is defaulted to 255.
   * 
   * @param r the red component.
   * @param g the green component.
   * @param b the blue component.
   */
  public Color(int r, int g, int b) {
    this(r, g, b, 255);
  }

  /**
   * Returns the red component in the range 0-255 in the default sRGB
   * space.
   * 
   * @return The red component.
   */
  public int getRed() {
    return (getRGB() >> 16) & 0xFF;
  }

  /**
   * Returns the green component in the range 0-255 in the default sRGB
   * space.
   * 
   * @return The green component.
   */
  public int getGreen() {
    return (getRGB() >> 8) & 0xFF;
  }

  /**
   * Returns the blue component in the range 0-255 in the default sRGB
   * space.
   * 
   * @return The blue component.
   */
  public int getBlue() {
    return (getRGB() >> 0) & 0xFF;
  }

  /**
   * Returns the alpha component in the range 0-255.
   * 
   * @return The alpha component.
   */
  public int getAlpha() {
    return (getRGB() >> 24) & 0xff;
  }
  
  /**
   * Returns the RGB value representing the color in the default sRGB
   * (Bits 24-31 are alpha, 16-23 are red, 8-15 are green, 0-7 are
   * blue).
   * 
   * @return the RGB value of the color in the default sRGB
   *         <code>ColorModel</code>.
   **/
  public int getRGB() {
    return rgb;
  }

  /**
   * Returns a string representation of this <code>Color</code>. This
   * method is intended to be used only for debugging purposes.  The 
   * content and format of the returned string might vary between 
   * implementations. The returned string might be empty but cannot 
   * be <code>null</code>.
   * 
   * @return A string representation of this <code>Color</code>.
   */
  @Override
  public String toString() {
    String colorInfo = getClass().getName() + "[r=" + getRed() + ",g=" + getGreen() + ",b=" + getBlue();
    if(getAlpha() != 255) {
      colorInfo += ",a=" + getAlpha();
    }
    colorInfo += "]";
    return colorInfo;
  }

  /**
   * Checks if the two colors have the same RGB value. The alpha is also taken into account.
   */
  @Override
  public boolean equals(Object obj) {
    boolean equals = false;
    if (obj instanceof Color) {
      Color otherColor = (Color) obj;
      equals = 
        otherColor.rgb == rgb;
    }
    return equals;
  }

  /**
   * @see java.lang.Object#hashCode()
   */
  @Override
  public int hashCode() {
    return rgb;
  }

  /**
   * Used for decrease the color saturation with the given percent.
   *
   * @param fillColor The color to be darkened.
   * @param percent The percent to be used for decrease the color saturation.
   * @return The modified color.
   */
  public static Color desaturate(Color fillColor, double percent) {
    int red = fillColor.getRed();
    int green = fillColor.getGreen();
    int blue = fillColor.getBlue();
    
    int newRed = (int) (red * percent);
    int newGreen = (int) (green * percent);
    int newBlue = (int) (blue * percent);
    
    Color color = new Color(newRed, newGreen, newBlue);
    return color;
  }

  /**
   * Used to simulate an alpha composition between a color and 
   * a background color.
   *
   * @param color The transparent color.
   * @param background The background color.
   * @return The resulted color.
   * @see "http://en.wikipedia.org/wiki/Alpha_compositing"
   */
  public static Color alphaComposite(Color color, Color background) {
    int red = color.getRed();
    int green = color.getGreen();
    int blue = color.getBlue();
    double alpha = color.getAlpha() / 255.0;
    
    int newRed = (int)(red * alpha +  background.getRed() * (1 - alpha));
    int newGreen = (int)(green * alpha +  background.getGreen() * (1 - alpha));
    int newBlue = (int)(blue * alpha +  background.getBlue() * (1 - alpha));
    
    return new Color(newRed, newGreen, newBlue);
  }

  
  /**
   * Method used to decode the given string to a color.
   *
   *@param  colorString  The string representation of the color.
   *@return              The decoded color if possible, black otherwise.
   */
  public static Color decodeColor(String colorString) {
    Color color = Color.COLOR_BLACK;
    if (colorString != null && colorString.length() > 0) {
      StringTokenizer stk = new StringTokenizer(colorString, ",", false);
      if (stk.countTokens() == 3 
       //Probably with alpha
      || stk.countTokens() == 4) {
        try {
          int red = NumberParserUtil.parseInt(stk.nextToken());
          int green = NumberParserUtil.parseInt(stk.nextToken());
          int blue = NumberParserUtil.parseInt(stk.nextToken());
          int alpha = 255;
          if(stk.hasMoreTokens()) {
            alpha = NumberParserUtil.parseInt(stk.nextToken());  
          }
          color = new Color(red, green, blue, alpha);
        } catch (NumberFormatException nfe) {
          //Do nothing, keep the default.
        }
      }
    }
    return color;
  }
  
  /**
   * RGB to HSl space convert method.
   * Copied from: http://www.easyrgb.com/math.php?MATH=M19#text19
   * 
   * @param r Red (0 -> 255)
   * @param g Green (0 -> 255)
   * @param b Blue (0 -> 255)
   * @return The HSL array of values between 0.0f and 1.0f
   */
  private static float[] rgbTohsl(int r, int g, int b) {
    float var_R = (((float) r) / 255); //Where RGB values = 0 to 255
    float var_G = (((float) g) / 255);
    float var_B = (((float) b) / 255);

    float var_Min = Math.min(Math.min(var_R, var_G), var_B); //Min. value of RGB
    float var_Max = Math.max(Math.max(var_R, var_G), var_B); //Max. value of RGB
    float del_Max = var_Max - var_Min; //Delta RGB value

    float L = (var_Max + var_Min) / 2;
    float H = 0;
    float S = 0;
    if (del_Max == 0) //This is a gray, no chroma...
    {
      H = 0; //HSL results = 0 to 1
      S = 0;
    } else //Chromatic data...
    {
      if (L < 0.5) {
        S = del_Max / (var_Max + var_Min);
      } else {
        S = del_Max / (2 - var_Max - var_Min);
      }

      float del_R = (((var_Max - var_R) / 6) + (del_Max / 2)) / del_Max;
      float del_G = (((var_Max - var_G) / 6) + (del_Max / 2)) / del_Max;
      float del_B = (((var_Max - var_B) / 6) + (del_Max / 2)) / del_Max;

      if (var_R == var_Max) {
        H = del_B - del_G;
      } else if (var_G == var_Max) {
        H = (1.0f / 3.0f) + del_R - del_B;
      } else if (var_B == var_Max) {
        H = (2.0f / 3.0f) + del_G - del_R;
      }

      if (H < 0){
        H += 1;
      } 
      if (H > 1) {
        H -= 1;
      }
    }
    return new float[] { H, S, L };
  }

  /**
   * HSL color space to RGB convert method.
   * Copied from: http://www.easyrgb.com/math.php?MATH=M19#text19
   * 
   * @param H (between 0.0 and 1.0)
   * @param S (between 0.0 and 1.0)
   * @param L (between 0.0 and 1.0)
   * @return The RGB color.
   */
  private static Color hslToRGB(float H, float S, float L) {
    int R = 0;
    int G = 0;
    int B = 0;
    if ( S == 0 )                       //HSL values = 0 to 1
    {
      R = (int) (L * 255);                      //RGB results = 0 to 255
      G = (int) (L * 255);
      B = (int) (L * 255);
    }
    else
    {
      float var_2;
      if ( L < 0.5 ) {
        var_2 = L * ( 1 + S );
      } else {
        var_2 = ( L + S ) - ( S * L );
      }
      float var_1 = 2 * L - var_2;
      R = (int) (255 * Hue_2_RGB( var_1, var_2, H + ( 1.0f / 3.0f ) ));
      G = (int) (255 * Hue_2_RGB( var_1, var_2, H ));
      B = (int) (255 * Hue_2_RGB( var_1, var_2, H - ( 1.0f / 3.0f ) ));
    }
    return new Color(R, G, B);
  }

  private static float Hue_2_RGB(float v1, float v2, float vH) {
    if ( vH < 0 ) {
      vH += 1;
    }
    if ( vH > 1 ) {
      vH -= 1;
    }
    if ( ( 6 * vH ) < 1 ) {
      return ( v1 + ( v2 - v1 ) * 6 * vH );
    }
    if ( ( 2 * vH ) < 1 ) {
      return ( v2 );
    }
    if ( ( 3 * vH ) < 2 ) {
      return ( v1 + ( v2 - v1 ) * ( ( 2.0f / 3.0f ) - vH ) * 6 );
    }
    return ( v1 );
  }
  
  /**
   * Decrease the brightness of the color.
   * 
   * @param color The color to be modified.
   * @return  The color with decreased brightness.
   */
  public static Color darker(Color color) {
    return darker(color, 0.5f);
  }

  /**
   * Decrease the brightness of the color.
   * 
   * @param color The color to be modified.
   * @param percent  The percent to be used for darkening a color.
   * @return  The color with decreased brightness.
   */
  public static Color darker(Color color, float percent) {
    float[] hsl = Color.rgbTohsl(color.getRed(), color.getGreen(), color.getBlue());
    hsl[2] = hsl[2] * percent;
    return Color.hslToRGB(hsl[0], hsl[1], hsl[2]);
  }

  
  /**
   * Increase brightness of the color.
   * 
   * @param color The color to be modified.
   * @return  The color with increased brightness.
   */
  public static Color brighter(Color color) {
    float[] hsl = Color.rgbTohsl(color.getRed(), color.getGreen(), color.getBlue());
    hsl[2] = (1.0f + hsl[2]) / 2;
    return Color.hslToRGB(hsl[0], hsl[1], hsl[2]);
  }
  
  /**
   * Gets the L component form the HSL spectrum of the color.
   * 
   * @return The brightness of the color. Between 0.0 and 1.0
   */
  public float getBrightness(){
    float[] hsl = Color.rgbTohsl(getRed(), getGreen(), getBlue());
    return hsl[2];    
  }
  
}