/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.workspace.api;

import java.io.File;
import java.io.Reader;
import java.net.URL;

import javax.swing.Action;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.ecss.dom.AuthorDocumentImpl;
import ro.sync.exml.EditorFrameDescriptor;
import ro.sync.exml.MainFrame;
import ro.sync.exml.OxygenSystemProperties;
import ro.sync.exml.editor.EditorManagerImpl;
import ro.sync.exml.options.LicenseOptions;
import ro.sync.exml.options.OptionTags;
import ro.sync.exml.options.Options;
import ro.sync.exml.workspace.api.editor.WSEditor;
import ro.sync.exml.workspace.api.editor.page.author.WSAuthorEditorPage;
import ro.sync.exml.workspace.api.standalone.StandalonePluginWorkspace;
import ro.sync.io.IOUtil;
import ro.sync.net.protocol.Installer;
import ro.sync.ui.UiUtil;
import ro.sync.util.PreferencesDirectoryDetector;
import ro.sync.util.URLUtil;


/**
 * Base class for testing plugins and frameworks.
 * For more details please read the topic called "Creating and Running Automated Tests" from the user manual:
 * <br>
 *  http://www.oxygenxml.com/doc/ug-oxygen/topics/automated-tests.html
 * 
 * @see "http://www.oxygenxml.com/doc/ug-oxygen/topics/automated-tests.html"
 * 
 * @since 14.1
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PRIVATE)
public abstract class PluginWorkspaceTCBase extends junit.extensions.jfcunit.JFCTestCase {
  
  /**
   * The main frame.
   */
  private MainFrame mainFrame;
  /**
   * The plugin workspace
   */
  private StandalonePluginWorkspace pluginWorkspace;
  
  /**
   * Constructor. The installation folder will be assumed to be the folder in which the JVM has started (new File(".")).
   * 
   * @param frameworksFolder The folder from where to load the frameworks. If <code>null</code> it will default to the folder "frameworks" in the installationFolder.
   * @param pluginsFolder  The folder from where to load the plugins. If <code>null</code> it will default to the folder "plugins" in the installationFolder.
   * @param licenseKey     The license key used to license the test Oxygen application.
   * @throws Exception 
   */
  public PluginWorkspaceTCBase(File frameworksFolder, File pluginsFolder, String licenseKey) throws Exception {
    this(null, frameworksFolder, pluginsFolder, null, licenseKey);
  }

  /**
   * Constructor.
   * @param installationFolder The folder where Oxygen is installed. If <code>null</code> it will default to the folder in which the JVM started (new File(".")).
   * @param frameworksFolder The folder from where to load the frameworks. If <code>null</code> it will default to the folder "frameworks" in the installationFolder.
   * @param pluginsFolder  The folder from where to load the plugins. If <code>null</code> it will default to the folder "plugins" in the installationFolder.
   * @param optionsFolder  The folder from where to load the Oxygen options. Set to null to use the default options folder on your specific platform (located in the user home). 
   * @param licenseKey     The license key used to license the test Oxygen application.
   * @throws Exception 
   */
  public PluginWorkspaceTCBase(File installationFolder, File frameworksFolder, File pluginsFolder, File optionsFolder, String licenseKey) throws Exception {
    //Set the license key.
    LicenseOptions.getInstance().setLicenseText(licenseKey);
    if(installationFolder == null) {
      //Instantiate to the current directory.
      installationFolder = new File(".");
    }
    //Home
    System.setProperty(OxygenSystemProperties.COM_OXYGENXML_EDITOR_HOME_URL, 
        URLUtil.correct(installationFolder).toString());
    
    if(pluginsFolder == null) {
      pluginsFolder = new File(installationFolder, "plugins");
    }
    //Set plugins directory
    System.setProperty(OxygenSystemProperties.COM_OXYGENXML_EDITOR_PLUGINS_DIR, pluginsFolder.getAbsolutePath());
    
    if(frameworksFolder != null) {
      //Set frameworks directory.
      System.setProperty(OxygenSystemProperties.COM_OXYGENXML_EDITOR_CUSTOM_FRAMEWORKS_URL, 
          URLUtil.correct(frameworksFolder).toString());
    } else {
      //The default frameworks folder will be used from the installation directory.
    }
    //Set a custom options folder
    if(optionsFolder != null) {
      PreferencesDirectoryDetector.setPrefsDir(optionsFolder.getAbsolutePath());
    }
    //Install protocols like in a standalone Oxygen.
    Installer.installProtocols();
    //Avoid some warnings
    Options.getInstance().setBooleanProperty(OptionTags.DO_NOT_SHOW_JVM_WARNING, true);
    Options.getInstance().setBooleanProperty(OptionTags.TIPOFTHEDAY_SHOW, false);
    Options.getInstance().setBooleanProperty(Options.SHOW_START_PAGE_DIALOG, false);
    //Create the main frame
    mainFrame = new MainFrame(new EditorFrameDescriptor(), null, null, null, true, false, null);
    pluginWorkspace = mainFrame.getPluginWorkspace();
  }
  
  /**
   * @see junit.extensions.jfcunit.JFCTestCase#tearDown()
   */
  @Override
  protected void tearDown() throws Exception {
    //Close all editors on tear down without saving.
    super.tearDown();
    ((EditorManagerImpl)mainFrame.getEditorManager()).closeAllWithoutSaveFromTCs();
    mainFrame.setVisible(false);
  }
  
  /**
   * @see junit.extensions.jfcunit.JFCTestCase#setUp()
   */
  @Override
  protected void setUp() throws Exception {
    super.setUp();
    mainFrame.setVisible(true);
  }
  
  /**
   * Get the plugin workspace.
   * 
   * @return Returns the plugin workspace.
   */
  public StandalonePluginWorkspace getPluginWorkspace() {
    return pluginWorkspace;
  }
  
  /**
   * Open an URL. 
   * 
   * @param url The URL to open.
   * @return The author page.
   * @throws Exception 
   */
  public WSEditor open(URL url) throws Exception {
    boolean success = pluginWorkspace.open(url);
    if(success) {
      return pluginWorkspace.getEditorAccess(url, StandalonePluginWorkspace.MAIN_EDITING_AREA);
    }
    return null;
  }
  
  /**
   * Get the XML content currently loaded in the current editor.
   * 
   * @return The XML content.
   * @throws Exception 
   */
  public String getCurrentEditorXMLContent() throws Exception {
    WSEditor editorAccess = pluginWorkspace.getCurrentEditorAccess(StandalonePluginWorkspace.MAIN_EDITING_AREA);
    Reader reader = editorAccess.createContentReader();
    return IOUtil.read(reader).toString();
  }
  
  /**
   * Get the WSAuthorEditorPage for the current edited file.
   * 
   * @return the WSAuthorEditorPage for the current edited file. Can throw class cast exception if the current editor is not opened in the author page.
   * @throws Exception 
   */
  public WSAuthorEditorPage getCurrentAuthorEditorPageAccess() throws Exception {
    WSEditor editorAccess = pluginWorkspace.getCurrentEditorAccess(StandalonePluginWorkspace.MAIN_EDITING_AREA);
    return (WSAuthorEditorPage) editorAccess.getCurrentPage();
  }
  
  /**
   * Invoke action with a certain ID on the AWT thread. 
   * @param id The id The action ID as defined in the framework.
   */
  protected void invokeAuthorExtensionActionForID(String id) throws Exception {
    final Action action = (Action) getCurrentAuthorEditorPageAccess().getActionsProvider().getAuthorExtensionActions().get(id);
    assertNotNull(action);
    UiUtil.invokeSynchronously(new Runnable() {
      @Override
      public void run() {
        action.actionPerformed(null);
      }
    });
  }
  
  /**
   * Move caret relative to a text already existing in the author page.
   * 
   * @param text The text to search
   * @param relativePosition The delta to move the caret with after finding the text.
   * @throws Exception
   */
  protected void moveCaretRelativeTo(
      String text,
      final int relativePosition,
      final boolean select) throws Exception {
    final WSAuthorEditorPage currentAuthorEditorPageAccess = getCurrentAuthorEditorPageAccess();
    String contentStr = ((AuthorDocumentImpl)currentAuthorEditorPageAccess.getDocumentController().getAuthorDocumentNode()).getText().replace('\0', 'X');
    final int idx = contentStr.indexOf(text);
    if (idx != -1) {
      UiUtil.invokeSynchronously(new Runnable() {
        @Override
        public void run() {
          if(select) {
            currentAuthorEditorPageAccess.select(currentAuthorEditorPageAccess.getCaretOffset(), idx + relativePosition);
          } else {
            currentAuthorEditorPageAccess.setCaretPosition(idx + relativePosition);
          }
        }
      });
      flushAWT();
    } else {
      throw new Exception("Text '" + text + "' cannot be found");
    }
  }
}
